
package diagrams;

import java.util.Enumeration;
import static java.lang.Math.floor;
import static java.lang.Math.log10;
import static java.lang.Math.pow;
import static java.lang.Math.round;

/**
 *
 * @author B. Gaertner
 */
public class LogarithmicScale extends AbstractScale {

    public final double base;
    private double logB;
    private double logOfLow;
    private double logOfHigh;
    private int lowerExp;
    private int upperExp;
    private int lowerCnt;
    private int upperCnt;

    public LogarithmicScale (double b, AbstractScaleRange sr, ScaleProperties sp) {
        super(sr, sp);
        base = b;
        logB = log10(base);
        double firstValue = sr.startValue;
        double lastValue = sr.endValue;
        logOfLow = log10(firstValue)/logB;
        logOfHigh = log10(lastValue)/logB;
        long logE1 = round(logOfLow);
        lowerExp = (int)(logOfLow < logE1 ? logE1 - 1 : logE1);
        upperExp = (int)(floor(logOfHigh));
        lowerCnt = (int)(firstValue/pow(10.0, lowerExp));
        upperCnt = (int)(lastValue/pow(10.0, upperExp));
        //int upperCnt2 = (int)(lastValue/pow(10.0, upperExp));
        lowerCnt = lowerCnt < 1 ? 1 : lowerCnt;
        if (upperCnt ==1) {
            upperExp -= 1;
            upperCnt = 10;
        }
    }

    /**
     *
     */
    public class LogarithmicScaleTickValueEnumeration implements Enumeration<TickValue> {
        private int localLowerCnt;
        private int currentExp;
        private int currentCnt;
        private double factor1;
        private int actualUpperCnt;

        public LogarithmicScaleTickValueEnumeration() {
            localLowerCnt = lowerCnt;
            currentExp = lowerExp;
            currentCnt = lowerCnt;
            factor1 = pow(10.0, currentExp);
            actualUpperCnt = currentExp == upperExp ? upperCnt : 9;
        }

        //  In den beiden folgenden MEthoden stecken noch einige Probleme
        @Override
        public boolean hasMoreElements() {
            return currentExp < upperExp
                    || (currentExp == upperExp && currentCnt - 1 < actualUpperCnt);
        }

        @Override
        public TickValue nextElement() {
            double value = currentCnt*factor1;
            TickValue tv = new TickValue (currentExp, currentCnt, value);
            currentCnt += 1;
            if (currentCnt > actualUpperCnt) {
                currentExp += 1;
                factor1 = pow(10.0, currentExp);
                actualUpperCnt = currentExp == upperExp ? upperCnt : 9;
                currentCnt = 1;
            }

            return tv;
        }
    }


    @Override
    public ScaleTransformation createTransformation(AbstractPaintRange sb) {
        return new LogarithmicTransformation
                (logB,
                 sb.paintWidth()/(logOfHigh - logOfLow),
                 logOfLow,
                 sb.paintOrigin());

    }

    @Override
    public Enumeration<TickValue> enumeration() {
        return new LogarithmicScaleTickValueEnumeration();
    }

    @Override
    public boolean tickIsLabelled(TickValue tv) {
        return tv.count < 5 || tv.count % 2 == 0;
    }
}
